
#define REL_REGBASE_SPI0	                      (0x00006000U)     /*!< SPI0 Master  */
#define REL_REGBASE_SPI_MST_CS_CTRL	              (0x00000014U)     /*!< SPI Master Select Ctrl */
#define REL_REGBASE_GPIO0						  (0x00002000U)		/*!< GPIO 0 Onboard */

typedef volatile struct dw_spi_reg {            /*!< Control Register */
    U32 CTRLR0;                                 /*!< SPI Control Register 0  (0x0) */
    U32 CTRLR1;                                 /*!< SPI Control Register 1  (0x4) */
    U32 SSIENR;                                 /*!< Enable Register */ /*!< SPI Enable Register  (0x8) */
    U32 MWCR;                                   /*!< SPI Microwire Control Register  (0xC) */
    U32 SER;                                    /*!< SPI Slave Enable Register  (0x10) */
    U32 BAUDR;                                  /*!< SPI Baud Rate Select Register  (0x14) */
    U32 TXFTLR;                                 /*!< TX and RX FIFO Control Register */	/*!< SPI Transmit FIFO Threshold Level Register  (0x18) */
    U32 RXFTLR;                                 /*!< SPI Receive  FIFO Threshold Level Register  (0x1C) */
    U32 TXFLR;                                  /*!< SPI Transmit FIFO Level Register  (0x20) */
    U32 RXFLR;                                  /*!< SPI Receive  FIFO Level Register  (0x24) */
    U32 SR;                                     /*!< SPI Status   Register  (0x28) */
    U32 IMR;                                    /*!< Interrupt Enable/Disable/Control Registers */ /*!< SPI Interrupt Mask Register  (0x2C) */
    U32 ISR;                                    /*!< SPI Interrupt Status Register  (0x30) */
    U32 RISR;                                   /*!< SPI Raw Interrupt Status Register (0x34) */
    U32 TXOICR;                                 /*!< SPI Transmit FIFO Overflow Interrupt Clear Register  (0x38) */
    U32 RXOICR;                                 /*!< SPI Receive  FIFO Overflow Interrupt Clear Register  (0x3C) */
    U32 RXUICR;                                 /*!< SPI Receive FIFO Underflow Interrupt Clear Register  (0x40) */
    U32 MSTICR;                                 /*!< SPI Multi-Master Interrupt Clear Register  (0x44) */
    U32 ICR;                                    /*!< SPI Interrupt Clear Register  (0x48) */
    U32 DMACR;                                  /*!< DMA Control Register  (0x4C) */
    U32 DMATDLR;                                /*!< DMA Transmit Data Level  (0x50) */
    U32 DMARDLR;                                /*!< DMA Receive Data Level  (0x54) */
    U32 IDR;                                    /*!< SPI Identification Register  (0x58) */
    U32 SSI_VER_ID;                             /*!< SPI CoreKit ID Register (Value after Reset : 0x3332322A)  (0x5C) */
    U32 DATAREG;                                /*!< Data Register */ /*!< SPI DATA Register for both Read and Write  (0x60) */
} DW_SPI_REG, *DW_SPI_REG_PTR; 


typedef struct port_ctrl {
    U32 DR;
    U32 DDR;
    U32 CTRL;
} PORT_CTRL;
typedef volatile struct dw_gpio_reg {
	PORT_CTRL SWPORTS[4];
    U32 INTEN;									/*!< (0x30) */
    U32 INTMASK;								/*!< (0x34) */
    U32 INTTYPE_LEVEL;							/*!< (0x38) */
    U32 INT_POLARITY;							/*!< (0x3c) */
    U32 INTSTATUS;								/*!< (0x40) */
    U32 RAW_INTSTATUS;							/*!< (0x44) */
    U32 DEBOUNCE;								/*!< (0x48) */
    U32 PORTA_EOI;								/*!< (0x4c) */
    U32 EXT_PORTS[4];							/*!< (0x50) -A      (0x54) -B        (0x58) -C        (0x5c) -D */
    U32 LS_SYNC;								/*!< (0x60) */
    U32 ID_CODE;								/*!< (0x64) */
    U32 RESERVED_3;								/*!< (0x68) */
    U32 VER_ID_CODE;							/*!< (0x6c) */
    U32 CONFIG_REG2;							/*!< (0x70) */
    U32 CONFIG_REG1;							/*!< (0x74) */
} DW_GPIO_REG, *DW_GPIO_REG_PTR;


#define DW_SPI_SR_DCOL                          (0x40)
#define DW_SPI_SR_TXE                           (0x20)
#define DW_SPI_SR_RFF                           (0x10)
#define DW_SPI_SR_RFNE                          (0x8)
#define DW_SPI_SR_TFE                           (0x4)
#define DW_SPI_SR_TFNF                          (0x2)
#define DW_SPI_SR_BUSY                          (0x1)
#define DW_SPI_SSI_ENABLE                       (1)
#define DW_SPI_SSI_DISABLE                      (0)

#define DW_SPI_CTRLR0_DFS_MASK                  (0xf)
#define DW_SPI_CTRLR0_SC_OFS                    (6)
#define DW_SPI_CTRLR0_SC_MASK                   (0xC0)
#define DW_SPI_CTRLR0_SCPH_HIGH                 (0x40)
#define DW_SPI_CTRLR0_SCPH_LOW                  (0)
#define DW_SPI_CTRLR0_SCPOL_HIGH                (0x80)
#define DW_SPI_CTRLR0_SCPOL_LOW                 (0)
#define DW_SPI_CTRLR0_TMOD_MASK                 (0x300)
#define DW_SPI_TMOD_TRANSMIT_RECEIVE            (0)
#define DW_SPI_TMOD_TRANSMIT_ONLY               (0x100)
#define DW_SPI_TMOD_RECEIVE_ONLY                (0x200)
#define DW_SPI_TMOD_EEPROM_READ_ONLY            (0x300)
#define DW_SPI_CTRLR0_FRF_MOTOROLA              (0x0)
#define DW_SPI_CTRLR0_FRF_TI                    (0x10)
#define DW_SPI_CTRLR0_FRF_MICROWIRE             (0x20)
#define DW_SPI_CTRLR0_SLV_OE_DISABLE            (1<<10)
#define DW_SPI_CTRLR0_SLV_OE_ENABLE             (0)

typedef enum spi_clk_mode {
	SPI_CPOL_0_CPHA_0 = 0,
	SPI_CPOL_0_CPHA_1 = 1,
	SPI_CPOL_1_CPHA_0 = 2,
	SPI_CPOL_1_CPHA_1 = 3,
	SPI_CLK_MODE_0    = SPI_CPOL_0_CPHA_0,
	SPI_CLK_MODE_1    = SPI_CPOL_0_CPHA_1,
	SPI_CLK_MODE_2    = SPI_CPOL_1_CPHA_0,
	SPI_CLK_MODE_3    = SPI_CPOL_1_CPHA_1
} SPI_CLK_MODE;

#define SPI_CLK_MODE_DEFAULT	                SPI_CLK_MODE_0
#define SPI_DFS_DEFAULT				            8

#define CLK_BUS_APB                             (25000000U)

//--------------------------------------------------------------------------
#define REL_REGBASE_PINMUX		(0x00000000U)		/*!< PINMUX */

typedef volatile U32        MUX_REG, MST_REG;

#define PMOD_MUX_CTRL	    0
#define I2C_MAP_CTRL	    1
#define SPI_MAP_CTRL	    2
#define UART_MAP_CTRL	    3

#define BIT0                (0)

//--------------------------------------------------------------------------

#define PM6_OFFSET          (20)
#define PM6_MASK		    (0xf << PM6_OFFSET)
#define PM6_UR_SPI_M0       ((1 << BIT0) << PM6_OFFSET)	  /*!< Pmod6[4:1] are connected to DW SPI Master signals using CS0_N */

//--------------------------------------------------------------------------

#define PM5_OFFSET          (16)
#define PM5_MASK		    (0xf << PM5_OFFSET)
#define PM5_UR_SPI_M0       ((1 << BIT0) << PM5_OFFSET)	  /*!< Pmod5[4:1] are connected to DW SPI Master signals using CS1_N */

//--------------------------------------------------------------------------

#define PM1_OFFSET          (0)
#define PM1_MASK            (0xf << PM1_OFFSET)
#define PM1_UR_UART_0       ((1 << BIT0) << PM1_OFFSET)	  /*!< Pmod1[4:1] are connected to DW UART0 signals */

//--------------------------------------------------------------------------
