/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                   SFS_Port_SPI_SAMD51.c                  *
*                   SAMD51 SPI xRAM PORT                   *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"

#ifdef __IAR_SYSTEMS_ICC__
 #include "Microchip/iosamd51.h"
#else
 #include <SAMD51.h>
#endif

#define SAMD51_TACHYON        0
#define SAMD51_ITSYBITSY_M4   1

#define SAMD51_TYPE           SAMD51_ITSYBITSY_M4

/***********************************************************
 *   Atmel SAMD51G18 configuration "TACHYON"
 ***********************************************************
 *   PB10 --- SERCOM4.2 ----------------------- MOSI    ->
 *   PB11 --- SERCOM4.3 ----------------------- SCK     ->
 *   PA13 --- SERCOM4.0 ----------------------- /CS     ->  (manual used)
 *   PA12 --- SERCOM4.1 ----------------------- MISO    <-
 **********************************************************/
#if (SAMD51_TYPE == SAMD51_TACHYON)
 #define SPI_SERCOMx   SERCOM4
 #define SPI_CS        PORT_PA13
#endif

/***********************************************************
 *   Atmel SAMD51G18 configuration "ItsyBitsy M4"
 ***********************************************************
 *   PA00 --- SERCOM1.0 ----------------------- MOSI    ->
 *   PA01 --- SERCOM1.1 ----------------------- SCK     ->
 *   PA07 --- --------- ----------------------- /CS     ->  (manual used)
 *   PB23 --- SERCOM1.3 ----------------------- MISO    <-
 **********************************************************/
#if (SAMD51_TYPE == SAMD51_ITSYBITSY_M4)
 #define SPI_SERCOMx   SERCOM1
 #define SPI_CS        PORT_PA07
#endif

#define SPI_MASTER_MODE   3

/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
    PORT->Group[0].OUTCLR.reg = SPI_CS;
}
/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
    PORT->Group[0].OUTSET.reg = SPI_CS;
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    while(!(SPI_SERCOMx->SPI.INTFLAG.reg & SERCOM_SPI_INTFLAG_DRE)) __NOP();
    SPI_SERCOMx->SPI.DATA.reg = byte;
    while(!(SPI_SERCOMx->SPI.INTFLAG.reg & SERCOM_SPI_INTFLAG_RXC)) __NOP();
    return (U08)SPI_SERCOMx->SPI.DATA.reg;
}

/*
************************************************************
*                LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08 sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}
/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    SPI_SERCOMx->SPI.CTRLA.bit.ENABLE = 0; 
#if (SAMD51_TYPE == SAMD51_TACHYON)
    MCLK->APBDMASK.reg |= MCLK_APBDMASK_SERCOM4; 
    GCLK->PCHCTRL[SERCOM4_GCLK_ID_CORE].reg = GCLK_PCHCTRL_GEN_GCLK1_Val | (1 << GCLK_PCHCTRL_CHEN_Pos); 
#endif
#if (SAMD51_TYPE == SAMD51_ITSYBITSY_M4)
    MCLK->APBAMASK.reg |= MCLK_APBAMASK_SERCOM1; 
    GCLK->PCHCTRL[SERCOM1_GCLK_ID_CORE].reg = GCLK_PCHCTRL_GEN_GCLK1_Val | (1 << GCLK_PCHCTRL_CHEN_Pos); 
#endif    
    SPI_SERCOMx->SPI.CTRLA.reg = SERCOM_SPI_CTRLA_SWRST;
    while(SPI_SERCOMx->SPI.SYNCBUSY.bit.SWRST) __NOP();
    SPI_SERCOMx->SPI.CTRLA.reg = SERCOM_SPI_CTRLA_DIPO(3) | SERCOM_SPI_CTRLA_DOPO(0) | SERCOM_SPI_CTRLA_MODE(SPI_MASTER_MODE);
    SPI_SERCOMx->SPI.BAUD.reg  = 2;   // from 100MHz and depends on the line-quality of SPI connection (loose wires or clear layout) and the used memory-HW
    SPI_SERCOMx->SPI.CTRLB.reg = SERCOM_SPI_CTRLB_RXEN;
    while(SPI_SERCOMx->SPI.SYNCBUSY.bit.CTRLB) __NOP();
#if (SAMD51_TYPE == SAMD51_TACHYON)
    PORT->Group[0].PMUX[PIN_PA12/2].bit.PMUXE      = 3;  // AF-D
    PORT->Group[0].PINCFG[PIN_PA12].bit.PMUXEN     = 1;
    PORT->Group[1].PMUX[(PIN_PB10-32)/2].bit.PMUXE = 3;  // AF-D
    PORT->Group[1].PINCFG[PIN_PB10-32].bit.PMUXEN  = 1;
    PORT->Group[1].PMUX[(PIN_PB11-32)/2].bit.PMUXO = 3;  // AF-D
    PORT->Group[1].PINCFG[PIN_PB11-32].bit.PMUXEN  = 1;
#endif
#if (SAMD51_TYPE == SAMD51_ITSYBITSY_M4)
    PORT->Group[0].PMUX[PIN_PA00/2].bit.PMUXE      = 3;  // AF-D
    PORT->Group[0].PINCFG[PIN_PA00].bit.PMUXEN     = 1;
    PORT->Group[0].PMUX[PIN_PA01/2].bit.PMUXO      = 3;  // AF-D
    PORT->Group[0].PINCFG[PIN_PA01].bit.PMUXEN     = 1;
    PORT->Group[1].PMUX[(PIN_PB23-32)/2].bit.PMUXO = 2;  // AF-C
    PORT->Group[1].PINCFG[PIN_PB23-32].bit.PMUXEN  = 1;
#endif
    SPI_SERCOMx->SPI.CTRLA.reg |= SERCOM_SPI_CTRLA_ENABLE;
    while(SPI_SERCOMx->SPI.SYNCBUSY.bit.ENABLE) __NOP();
     // config the additional manual line PAxx as /CS
    PORT->Group[0].DIRSET.reg = SPI_CS;
    PORT->Group[0].OUTSET.reg = SPI_CS;
}

/************************* END ****************************/
