/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                   SFS_Port_SPI_SAML10.c                  *
*                   SAML10 SPI xRAM PORT                   *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"

#include <sam.h>

/***********************************************************
 *   Atmel SAML10E16 configuration
 ***********************************************************
 *   PA16 --- SERCOM1.0 ----------------------- MOSI    ->
 *   PA17 --- SERCOM1.1 ----------------------- SCK     ->
 *   PA18 --- SERCOM1.2 ----------------------- /CS     ->  (manual used)
 *   PA19 --- SERCOM1.3 ----------------------- MISO    <-
 **********************************************************/

/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
    PORT->Group[0].OUTCLR.reg = PORT_PA18;
}
/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
    PORT->Group[0].OUTSET.reg = PORT_PA18;
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    while(!(SERCOM1->SPI.INTFLAG.reg & SERCOM_SPI_INTFLAG_DRE)) __NOP();
    SERCOM1->SPI.DATA.reg = byte;
    while(!(SERCOM1->SPI.INTFLAG.reg & SERCOM_SPI_INTFLAG_RXC)) __NOP();
    return (U08)SERCOM1->SPI.DATA.reg;
}

/*
************************************************************
*                LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08 sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}
/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    MCLK->APBCMASK.reg |= MCLK_APBCMASK_SERCOM1;
    GCLK->PCHCTRL[SERCOM1_GCLK_ID_CORE].reg = GCLK_PCHCTRL_GEN(0) | GCLK_PCHCTRL_CHEN;
    while (0 == (GCLK->PCHCTRL[SERCOM1_GCLK_ID_CORE].reg & GCLK_PCHCTRL_CHEN));
    SERCOM1->SPI.CTRLA.reg = SERCOM_SPI_CTRLA_SWRST;
    while(SERCOM1->SPI.SYNCBUSY.bit.SWRST) __NOP();
    SERCOM1->SPI.CTRLA.reg = SERCOM_SPI_CTRLA_DIPO(3) | SERCOM_SPI_CTRLA_DOPO(0) | SERCOM_SPI_CTRLA_MODE(3);
    SERCOM1->SPI.BAUD.reg  = 0;           // depends on the line-quality of SPI connection (loose wires or clear layout) and the used memory-HW
    SERCOM1->SPI.CTRLB.reg = SERCOM_SPI_CTRLB_RXEN;
    while(SERCOM1->SPI.SYNCBUSY.bit.CTRLB) __NOP();
    PORT->Group[0].PINCFG[PIN_PA16].bit.PMUXEN = 1;
    PORT->Group[0].PINCFG[PIN_PA17].bit.PMUXEN = 1;
    PORT->Group[0].PINCFG[PIN_PA19].bit.PMUXEN = 1;
    PORT->Group[0].PMUX[PIN_PA16/2].reg = (PORT_PMUX_PMUXE(2) | PORT_PMUX_PMUXO(2));
    PORT->Group[0].PMUX[PIN_PA18/2].reg = (PORT_PMUX_PMUXO(2));
    SERCOM1->SPI.CTRLA.reg |= SERCOM_SPI_CTRLA_ENABLE;
    while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) __NOP();
     // config the additional manual line /CS
    PORT->Group[0].DIRSET.reg = PORT_PA18;
    PORT->Group[0].OUTSET.reg = PORT_PA18;
}

/************************* END ****************************/
